from typing import List


class InvalidAWSClientException(Exception):
    def __init__(self, client_name: str, allowed_vals: List[str]):
        self.message = f"Client {client_name} is not a valid AWS client. Allowed values are {allowed_vals}"

    def __str__(self):
        return self.message


class PdfCreationException(Exception):
    def __init__(self, survey_id: str):
        self.message = (
            f"An exception occurred while creating pdf for survey {survey_id}."
        )

    def __str__(self):
        return self.message


class S3UploadException(Exception):
    def __init__(self, bucket_name: str, object_id: str):
        self.message = (
            f"An exception occurred while trying to upload object {object_id} "
            f"to S3 bucket {bucket_name}."
        )

    def __str__(self):
        return self.message



from typing import Any, Callable
from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_CognitoIDP
from navalmartin_mir_aws_utils.boto3_client import get_aws_cognito_idp_client
from navalmartin_mir_aws_utils.utils import (
    AWSCognitoSignUpUserData,
    AWSCognitoSignInUserData,
)
from navalmartin_mir_aws_utils.utils import get_secret_hash


def global_signout_user_from_pool(
    access_token: str,
    credentials: AWSCredentials_CognitoIDP,
    cognito_client: Any = None,
) -> Any:
    """Signs out users from all devices.
    It also invalidates all refresh tokens that Amazon Cognito
    has issued to a user. A user can still use a hosted UI cookie to
    retrieve new tokens for the duration of the 1-hour cookie validity period.

    Parameters
    ----------
    cognito_client: The cognito client to use
    access_token: The access token that corresponds to the signed in  user
    credentials: Credentials for accessing the Cognito IDP service

    """

    if cognito_client is None:
        cognito_client = get_aws_cognito_idp_client(credentials=credentials)

    return cognito_client.global_sign_out(AccessToken=access_token)


def signup_cognito_user(
    user_data: AWSCognitoSignUpUserData,
    aws_cognito_credentials: AWSCredentials_CognitoIDP,
    secret_hash_builder: Callable,
    cognito_client: Any = None,
    print_exception_info: bool = False,
) -> Any:
    """Signup the user on the AWS Cognito

     Parameters
    ----------
    user_data: The user data to signup
    aws_cognito_credentials: Credentials for accessing the Cognito IDP service
    secret_hash_builder: The callable to build the secret hash
    print_exception_info: Flag indicating if information is to be printed upon excpetion

    """
    try:
        aws_cognito_user_client_id = aws_cognito_credentials.aws_cognito_pool_id
        aws_cognito_client_secret = aws_cognito_credentials.aws_cognito_client_secret

        secret_hash = secret_hash_builder(
            user_data.email, aws_cognito_user_client_id, aws_cognito_client_secret
        )

        if cognito_client is None:
            cognito_client = get_aws_cognito_idp_client(
                credentials=aws_cognito_credentials
            )

        cognito_resp = cognito_client.sign_up(
            ClientId=aws_cognito_user_client_id,
            SecretHash=secret_hash,
            Username=user_data.email,
            Password=user_data.password,
            UserAttributes=[
                {"Name": "family_name", "Value": user_data.surname},
                {"Name": "name", "Value": user_data.name},
            ],
        )
        return cognito_resp, secret_hash
    except Exception as e:
        print("Signup AWS Cognito failed")

        if print_exception_info:
            print(f"AWS Region name {aws_cognito_credentials.aws_region}")
            print(
                f"AWS Cognito client Id {aws_cognito_credentials.aws_cognito_client_id}"
            )
            print(
                f"AWS Cognito client secret {aws_cognito_credentials.aws_cognito_client_secret}"
            )
            print(f"Exception message {str(e)}")
        raise e


def authenticate_and_get_token_for_user(
    user_data: AWSCognitoSignInUserData,
    aws_cognito_credentials: AWSCredentials_CognitoIDP,
    cognito_client: Any = None,
) -> Any:
    if cognito_client is None:
        cognito_client = get_aws_cognito_idp_client(credentials=aws_cognito_credentials)

    secret_hash = get_secret_hash(
        username=user_data.username,
        client_id=aws_cognito_credentials.aws_cognito_client_id,
        client_secret=aws_cognito_credentials.aws_cognito_client_secret,
    )

    resp = cognito_client.initiate_auth(
        ClientId=aws_cognito_credentials.aws_cognito_client_id,
        AuthFlow="USER_PASSWORD_AUTH",
        AuthParameters={
            "USERNAME": user_data.username,
            "PASSWORD": user_data.password,
            "SECRET_HASH": secret_hash,
        },
    )
    return resp



from navalmartin_mir_aws_utils.aws_credentials import AWSCredentials_S3
from navalmartin_mir_aws_utils import (
    delete_s3_all_objs_with_key,
    delete_s3_object_with_key,
    generate_presigned_url
)

AWS_REGION = "YOUR-AWS-REGION"
AWS_S3_BUCKET_NAME = "YOUR-AWS-BUCKET-NAME"

if __name__ == "__main__":

    # credentials so that we can access S3
    aws_s3_credentials = AWSCredentials_S3(
        aws_s3_bucket_name=AWS_S3_BUCKET_NAME,
        aws_region=AWS_REGION
    )

    print(aws_s3_credentials)
    print("---------Deleting one object")

    # delete an object from S3
    key = "your-key-to-delete"
    delete_response = delete_s3_object_with_key(
        key=key, s3_client=None, aws_creds=aws_s3_credentials
    )

    print(delete_response)
    print("---------Deleting multiple objects")
    keys = [
        {"Key": "key-1-to-delete"},
        {"Key": "key-2-to-delete"},
    ]
    delete_response = delete_s3_all_objs_with_key(
        keys=keys, s3_client=None, aws_creds=aws_s3_credentials
    )
    print(delete_response)

    key = "your-key-to-access"
    response = generate_presigned_url(aws_creds=aws_s3_credentials,
                                      object_name=key,
                                      client_method_name='get_object',
                                      expiration=4600)
    print(response)



import os
from typing import List


def get_or_raise_environment_vars(var_names: List[str]) -> dict:
    """Loops over the given variable names and checks the environment
    about its value. If the value is None it raises an exception

    Parameters
    ----------
    var_names: The variable names to look for

    Returns
    -------

    A dictionary with the values of the environment variables
    """

    variables = {}

    for name in var_names:
        if name in variables:
            raise ValueError(f"Duplicate variable {name} found")
        value = os.getenv(name, default=None)

        if value is None:
            raise ValueError(f"Environment variable {name} is not set")

        variables[name] = value

    return variables



from navalmartin_mir_aws_utils import AWSCredentials_SFN
from navalmartin_mir_aws_utils.boto3_client import get_aws_sfn_client


def start_sfn_execution(
    aws_sfn_credentials: AWSCredentials_SFN, sfn_input: str, **kwargs
) -> dict:
    """Starts the execution of the state machine
    specified by the ARN in the provided AWSCredentials_SFN credentials.
    See here https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/stepfunctions/client/start_execution.html
    on how to specify the sfn_input. This must be given even if
    no input is required

    Parameters
    ----------

    aws_sfn_credentials: AWS credentials to access the
    sfn_input: The input to start the execution

    Returns
    -------

    A dictionary with the response of t
    """

    if "stateMachineArn" in kwargs:
        raise ValueError("stateMachineArn should not be specified in kwargs")

    if "input" in kwargs:
        raise ValueError("stateMachineArn should not be specified in kwargs")

    sfn_client = get_aws_sfn_client(credentials=aws_sfn_credentials)
    return sfn_client.start_execution(
        stateMachineArn=aws_sfn_credentials.state_machine_arn, input=sfn_input, **kwargs
    )



from typing import Any
from pydantic import EmailStr
from navalmartin_mir_aws_utils import AWSCredentials_SES
from navalmartin_mir_aws_utils import get_aws_client_factory
from navalmartin_mir_aws_utils.utils import Email


def verify_email_identity(email: EmailStr, credentials: AWSCredentials_SES, ses_client: Any = None) -> dict:
    """Before we can send an email using SES, we need to verify the email address

    Parameters
    ----------
    email: The email to verify
    credentials: The credentials to use to connect to the AWS SES client
    ses_client: Use the ses client provided by the client code

    Returns
    -------
    A dictionary describing the client response
    """

    if ses_client is None:
        ses_client = get_aws_client_factory(credentials=credentials)

    response = ses_client.verify_email_identity(EmailAddress=email)
    return response


def send_simple_email(
        email: Email,
        credentials: AWSCredentials_SES, ses_client: Any = None
) -> dict:
    """Send a simple text-based email. If the email body contains
    HTML use the send_html_email method below

    Parameters
    ----------
    email
    credentials
    ses_client

    Returns
    -------

    """
    if ses_client is None:
        ses_client = get_aws_client_factory(credentials=credentials)

    reply_emails = []
    if email.reply_to is not None:
        reply_emails = [email.reply_to]

    # format the email and send it
    response = ses_client.send_email(
        Source=email.source,
        Destination={
            "ToAddresses": [
                email.destination,
            ],
            "CcAddresses": email.ccs,
        },
        ReplyToAddresses=reply_emails,
        Message={
            "Subject": {"Data": email.subject.data, "Charset": email.charset},
            "Body": {"Text": {"Data": email.body.data, "Charset": email.charset}},
        },
    )
    return response


def send_html_email(email: Email,
                    credentials: AWSCredentials_SES,
                    ses_client: Any = None) -> dict:
    """

    Parameters
    ----------
    email
    credentials
    ses_client

    Returns
    -------

    """
    if ses_client is None:
        ses_client = get_aws_client_factory(credentials=credentials)

    reply_emails = []
    if email.reply_to is not None:
        reply_emails = [email.reply_to]

    # format the email and send it
    response = ses_client.send_email(
            Source=email.source,
            Destination={
                "ToAddresses": [
                    email.destination,
                ],
                "CcAddresses": email.ccs,
            },
            ReplyToAddresses=reply_emails,
            Message={
                "Subject": {"Data": email.subject.data, "Charset": email.charset},
                "Body": {"Html": {"Data": email.body.data, "Charset": email.charset}},
            },
        )
    return response



from navalmartin_mir_aws_utils import AWSCredentials_SageMaker
from navalmartin_mir_aws_utils import get_aws_client_factory

if __name__ == '__main__':

    sagemaker_credentials = AWSCredentials_SageMaker(aws_region="eu-west-2")
    client = get_aws_client_factory(credentials=sagemaker_credentials)

    # create the model
    args = {'ModelName': "protonet_resnet18_vessel_classifier",
            "PrimaryContainer":{
                            'ContainerHostname': 'string',
                            'Image': 'string',
                            'ImageConfig': {
                                       'RepositoryAccessMode': 'Platform',
                                       'RepositoryAuthConfig': {
                                           'RepositoryCredentialsProviderArn': 'string'
                                       }
                            },
                            'Mode': 'SingleModel' | 'MultiModel',
                            'ModelDataUrl': 'string',
                            'Environment': {
                                       'string': 'string'
                            },
                            'ModelPackageName': 'string',
                                   'InferenceSpecificationName': 'string',
                                   'MultiModelConfig': {
                                       'ModelCacheSetting': 'Enabled' | 'Disabled'
                                   }
                            },
    }
    model_creation = client.create_model(**args)


from pydantic import BaseModel, Field, EmailStr


class AWSCognitoSignUpUserData(BaseModel):
    email: EmailStr = Field(
        title="email", description="The email of the user to sign-up"
    )
    password: str = Field(
        title="password", description="The password of the user to sign-up"
    )
    name: str = Field(title="name", description="The name of the user to sign-up")
    surname: str = Field(
        title="surname", description="The surname of the user to sign-up"
    )


class AWSCognitoSignInUserData(BaseModel):
    username: EmailStr = Field(
        title="username", description="The username of the user to sign-in"
    )
    password: str = Field(
        title="password", description="The password of the user to sign-in"
    )



import asyncio
from navalmartin_mir_aws_utils.tasks_lambdas import aws_lambda_task_decorator


def before_task_runner(args: dict):
    print(f"Running before_task_runner with arguments {args}")
    return "Finished before task runner"


def after_task_runner(args: dict):
    print(f"Running after_task_runner with arguments {args}")
    return "Finished after task runner"


@aws_lambda_task_decorator(
    before_task_runner=before_task_runner, after_task_runner=after_task_runner
)
def task(args: dict):
    print(f"Running task with arguments {args}")
    return "None"


if __name__ == "__main__":
    result = task({})
    print(result)
